unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    GoCheck: TCheckBox;
    GenTime: TTimer;
    LogMemo: TMemo;
    StartStopTime: TTimer;
    GroupBox1: TGroupBox;
    StartStopLabel: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    StartStopCheck: TCheckBox;
    ActiveEdit: TEdit;
    InactiveEdit: TEdit;
    IntervalShiftCheck: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure GoCheckClick(Sender: TObject);
    procedure GenTimeTimer(Sender: TObject);
    procedure StartStopCheckClick(Sender: TObject);
    procedure StartStopTimeTimer(Sender: TObject);
    procedure IntervalShiftCheckClick(Sender: TObject);
  protected
    DI: LongWord;
    DataClock: Byte;
    bProgrammaticChange: Boolean;
    StartStopClock: LongWord;
    bGo: Boolean;
    ActiveTime, InactiveTime: Integer;
    TicksPerSec, NextShiftTick: Int64;
    bFast: Boolean;

    procedure DoFail(Error: LongWord; Source: String);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  AIOUSB,
  HWave;

const
  FullCircle = Pi * 2;
  NumCh = 3;

procedure TForm1.DoFail(Error: LongWord; Source: String);
var
  bBak: Boolean;

begin
  bBak := bProgrammaticChange;
  try
    bProgrammaticChange := True;

    GoCheck.Checked := False;
    LogMemo.Lines.Add('Error ' + IntToStr(Error) + ' from ' + Source + '.');

  finally
    bProgrammaticChange := bBak;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  //`@
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  GoCheck.Checked := False;
end;

procedure TForm1.GoCheckClick(Sender: TObject);
var
  Status: LongWord;
  DACData: array of TDACEntry;
  iCh: Integer;

begin
  if bProgrammaticChange then Exit;
  try
    bProgrammaticChange := True;

    SetLength(DACData, NumCh);
    for iCh := 0 to NumCh-1 do begin
      DACData[iCh].Channel := iCh;
      DACData[iCh].Value := $0000;
    end;

    if GoCheck.Checked then begin
      AIOUSB_ReloadDeviceLinks;
      DI := ResolveDeviceIndex(diOnly);
      if DI = diNone then begin
        DoFail(GetLastError, 'diOnly');
        Exit;
      end;

      DACMultiDirect(DI, @DACData[0].Channel, NumCh);
      DACSetBoardRange(DI, 1);

      bFast := False;
      Status := HWave_Start(DI, 2.5, 25);
      if Status <> ERROR_SUCCESS then begin
        DoFail(Status, 'HWave_Start');
        Exit;
      end;

      if IntervalShiftCheck.Checked then IntervalShiftCheckClick(IntervalShiftCheck);

      {
      SetLength(DataClock, NumCh);
      for iCh := 0 to NumCh-1 do
        DataClock[iCh] := 0
      ;
      }
      DataClock := 1;
      GenTime.Enabled := True;
    end
    else begin
      GenTime.Enabled := False;

      Status := HWave_Stop(DI);

      DACMultiDirect(DI, @DACData[0].Channel, NumCh);

      if Status <> ERROR_SUCCESS then begin
        DoFail(Status, 'HWave_Stop');
        Exit;
      end;
    end;

  finally
    bProgrammaticChange := False;
  end;
end;

procedure TForm1.GenTimeTimer(Sender: TObject);
var
  DACData: array of TDACEntry;
  iCh: Integer;
  Th, F: Extended;
  Status: LongWord;
  Naui: Int64;
  Milliseconds: Double;

begin
  if not GenTime.Enabled then Exit;

  SetLength(DACData, NumCh);
  repeat
    Th := FullCircle * Frac((DataClock shr 1) / 8);
    F := (3/4 - 1/4*Cos(Th)) * (DataClock and 1);
    F := 1/2 + 1/2*F;
    for iCh := 0 to NumCh-1 do begin
      //F := 1/2 + 1/2*Sin(FullCircle * DataClock[iCh]);

      DACData[iCh].Channel := iCh;
      DACData[iCh].Value := Round($FFFF * F);
    end;

    Status := HWave_QueueMulti(DI, @DACData[0], NumCh);
    if Status = ERROR_NOT_READY then Break;

    if Status <> ERROR_SUCCESS then begin
      DoFail(Status, 'HWave_QueueMulti');
      Exit;
    end;

    {
    for iCh := 0 to NumCh-1 do begin
      DataClock[iCh] := DataClock[iCh] + 9/17 / (1+iCh);
      if DataClock[iCh] > 1 then DataClock[iCh] := DataClock[iCh] - 1;
    end;
    }
    Inc(DataClock);
  until False;

  if IntervalShiftCheck.Checked then begin
    QueryPerformanceCounter(Naui);
    if (Naui - NextShiftTick) >= 0 then begin
      NextShiftTick := NextShiftTick + 60*TicksPerSec;
      bFast := not bFast;
      if bFast then
        Milliseconds := 1
      else
        Milliseconds := 2.5
      ;
      Status := HWave_ChangeClockRate(DI, Milliseconds);
      if Status <> ERROR_SUCCESS then begin
        DoFail(Status, 'HWave_ChangeClockRate');
        Exit;
      end;
    end;
  end;
end;

procedure TForm1.StartStopCheckClick(Sender: TObject);
begin
  if StartStopCheck.Checked then begin
    ActiveEdit.Enabled := False;
    InactiveEdit.Enabled := False;
    ActiveTime := StrToInt(ActiveEdit.Text);
    InactiveTime := StrToInt(InactiveEdit.Text);

    bGo := False;
    StartStopClock := InactiveTime;
    StartStopTime.Enabled := True;
    StartStopLabel.Caption := IntToStr(StartStopClock);
  end
  else begin
    StartStopTime.Enabled := False;
    GoCheck.Checked := False;
    StartStopLabel.Caption := '';

    ActiveEdit.Enabled := True;
    InactiveEdit.Enabled := True;
  end;
end;

procedure TForm1.StartStopTimeTimer(Sender: TObject);
begin
  if not StartStopTime.Enabled then Exit;

  if bGo and not GoCheck.Checked then begin
    StartStopCheck.Checked := False;
    Exit;
  end;

  Dec(StartStopClock);
  StartStopLabel.Caption := IntToStr(StartStopClock);
  if StartStopClock > 0 then Exit;

  bGo := not bGo;
  GoCheck.Checked := bGo;
  if bGo then begin
    StartStopClock := ActiveTime;
  end
  else begin
    StartStopClock := InactiveTime;
  end;
  StartStopLabel.Caption := IntToStr(StartStopClock);

  //`@
end;

procedure TForm1.IntervalShiftCheckClick(Sender: TObject);
begin
  QueryPerformanceFrequency(TicksPerSec);
  QueryPerformanceCounter(NextShiftTick);
  NextShiftTick := NextShiftTick + 60*TicksPerSec;
end;

end.
